/* Author : Dario GENGA
 * Date : 23.11.2021
 * Description : Library to manipulate the stack
 */

#include "stack.h"
#include <stdio.h>

void stack_init(stack *s, int size) {
    s->data = malloc(size * sizeof(int));
    s->size = size;
    s->top = -1;
}

bool stack_is_empty(stack s) {
    bool is_empty = true;

    if (s.top >= 0 && s.size >= 0 && s.data[0] >= 0) {
        is_empty = false;
    }

    return is_empty;
}

void stack_push(stack *s, int val) {
    int next_top = s->top + 1;

    // Push the value if the stack is not full
    if (next_top < s->size && s->data[next_top] < 0) {
        s->data[next_top] = val;
        s->top = next_top;
    }
}

void stack_pop(stack *s, int *val) {
    if (s->top >= 0) {
        *val = s->data[s->top];
        s->data[s->top] = -1;
        s->top -= 1;
    }
}

void stack_peek(stack s, int *val) {
    if (s.size > 0) {
        *val = s.data[s.top];
    }
}

void stack_destroy(stack *s) {
    free(s->data);
    s->data = NULL;
    s->top = -1;
    s->size = -1;
}

int *sort(int unsorted_array[], int array_size) {
    int *sorted_array = malloc(array_size * sizeof(int));
    int *left_peak = malloc(sizeof(int));
    int *right_peak = malloc(sizeof(int));
    stack left_stack, right_stack;

    stack_init(&left_stack, array_size);
    stack_init(&right_stack, array_size);

    for (int i = 0; i < array_size; i++) {
        int value = unsorted_array[i];

        // First stack push
        if (stack_is_empty(left_stack)) {
            stack_push(&left_stack, value);
        }
            // Add to left stack if the value is lower than the left peek and higher than the right peak
        else {
            stack_peek(left_stack, left_peak);
            if (*left_peak > value && (right_stack.top < 0 || *right_peak < value)) {
                stack_push(&left_stack, value);
            }
            // Else arrange the stacks
            else {
                while (left_stack.top >= 0 && *left_peak < value) {
                    stack_pop(&left_stack, left_peak);
                    stack_push(&right_stack, *left_peak);
                    stack_peek(left_stack, left_peak);
                    stack_peek(right_stack, right_peak);
                }
                while (right_stack.top >= 0 && *right_peak > value) {
                    stack_pop(&right_stack, right_peak);
                    stack_push(&left_stack, *right_peak);
                    stack_peek(right_stack, right_peak);
                }
                // Then push the value to the left stack
                stack_push(&left_stack, value);
            }
        }
    }

    // Push to right stack to the left stack
    while (right_stack.top >= 0) {
        stack_peek(right_stack, right_peak);
        stack_pop(&right_stack, right_peak);
        stack_push(&left_stack, *right_peak);
    }

    // Add the stack to the array
    for (int i = 0; i < array_size; i++) {
        stack_pop(&left_stack, left_peak);
        sorted_array[i] = *left_peak;
    }

    // Free the memory
    free(left_peak);
    free(right_peak);
    stack_destroy(&left_stack);
    stack_destroy(&right_stack);

    return sorted_array;
}