/* Author : Dario GENGA
 * Date : 23.11.2021
 * Description : Library to manipulate the stack
 */
#include "stack.h"
#include <assert.h>
#include <stdbool.h>
#include <stdio.h>
#define ARRAY_SIZE 5

void test_stack_init(stack *s, int size) {
    printf("Testing stack_init()...\n");
    stack_init(s, size);
    assert(s != NULL);
    assert(s->size == size);
    printf("OK.\n");
}

void test_stack_destroy(stack *s) {
    printf("Testing stack_destroy()...\n");
    stack_destroy(s);
    assert(s->data == NULL);
    assert(s->top == -1);
    printf("OK.\n");
}

void test_stack_is_empty(stack *s) {
    printf("Testing stack_is_empty()...\n");
    assert(stack_is_empty(*s) == true);
    printf("OK.\n");
}

void test_stack_is_not_empty(stack *s) {
    printf("Testing stack_is_not_empty()...\n");
    assert(stack_is_empty(*s) == false);
    printf("OK.\n");
}

void test_stack_peek(stack s, int *val) {
    printf("Testing stack_peek()...\n");
    stack_peek(s, val);
    // TODO: Verify test
    assert(*val == s.data[s.top]);
    printf("OK.\n");
}

void test_stack_push(stack *s, int val) {
    printf("Testing stack_push()...\n");
    stack_push(s, val);
    // Testing push with remaining spaces
    assert(s->data[s->top] == val);
    // Testing push on full stack
    stack stack_full;
    int value_top_full_stack_value = 100;
    stack_init(&stack_full, 1);
    stack_push(&stack_full, value_top_full_stack_value);
    stack_push(&stack_full, val);
    assert(stack_full.data[stack_full.top] == value_top_full_stack_value);
    printf("OK.\n");
    stack_destroy(&stack_full);
}

void test_stack_pop(stack *s, int *val) {
    printf("Testing stack_pop()...\n");
    int top_before_pop;
    // Test with non-empty stack
    if (s-> size == 0) {
        // Add a value if the stack is empty
        stack_push(s, 5);
    }
    top_before_pop = s->data[s->top];
    stack_pop(s, val);
    assert(*val == top_before_pop);

    // Clear the stack
    while(s->top >= 0) {
        stack_pop(s, val);
    }
    // Test pop when stack is empty
    val = NULL;
    stack_pop(s, val);
    assert(val == NULL);

    printf("OK.\n");
}

void test_sort(int unsorted_array[], int expected_array[], int array_size) {
    printf("Testing sort()...\n");
    int* sorted_array = sort(unsorted_array, array_size);

    for (int i = 0; i < array_size; i++) {
        assert(sorted_array[i] == expected_array[i]);
    }

    printf("OK.\n");
    free(sorted_array);
}

int main() {
    stack s, non_empty_stack;
    int size = 10, value_to_add = 7;
    int val;
    int unsorted_array[ARRAY_SIZE] = { 0, -1, 2, 7, 4 };
    int expected_array[ARRAY_SIZE] = { -1, 0, 2, 4, 7 };

    printf("Starting the tests...\n");
    // Test init and destroy
    test_stack_init(&s, size);
    test_stack_destroy(&s);

    // Reinitialization
    stack_init(&s, size);
    stack_init(&non_empty_stack, 3);
    non_empty_stack.data[0] = 1;
    non_empty_stack.top = 0;

    // Test the other functions
    test_stack_is_empty(&s);
    test_stack_is_not_empty(&non_empty_stack);
    test_stack_peek(s, &val);
    test_stack_push(&s, value_to_add);
    test_stack_pop(&s, &val);
    test_sort(unsorted_array, expected_array, ARRAY_SIZE);

    // Free the memory
    stack_destroy(&s);
    stack_destroy(&non_empty_stack);

    // End the tests
    printf("The tests are completed and were successful !\n");
    return EXIT_SUCCESS;
}
